/*************************************/
/* Fichier: rotation_matrix.c        */
/* Version: 2.0                      */
/* Date:   July 2011                 */
/* Author: Didier Pinchon            */
/*************************************/
#include "rotation.h"

int   matl_matl_product(const MatL *matl1, const MatL *matl2, MatL *matres)
{
   int l;
   if (matl1->lmax!=matl2->lmax) 
     fprintf(stderr,"Dimension error in matl_matl_product!\n");
   
   for (l=0;l<=matl1->lmax;l++)
     gsl_blas_dgemm(CblasNoTrans,CblasNoTrans, 1.0, matl1->lmat[l], matl2->lmat[l], 0.0, matres->lmat[l]);
   return 0;
}

int   Zalpha_matl_product(const MatL *matl, const double alpha,MatL *res) 
{
   int l,i,j;
   double co,si,x,y;
   extern double cos(double);
   extern double sin(double);

   for (l=0;l<=matl->lmax;l++)
      for (j=0;j<=2*l;j++) 
          gsl_matrix_set(res->lmat[l],l,j,gsl_matrix_get(matl->lmat[l],l,j));
        
   for (i=1;i<=matl->lmax;i++) {
       co = cos(i*alpha);
       si = sin(i*alpha);
       for (l=i;l<=matl->lmax;l++) {
	  for (j=0;j<=2*l;j++) {
	    x = gsl_matrix_get(matl->lmat[l],l-i,j);
	    y = gsl_matrix_get(matl->lmat[l],l+i,j);
	    gsl_matrix_set(res->lmat[l],l-i,j, co*x+si*y);
	    gsl_matrix_set(res->lmat[l],l+i,j, -si*x+co*y);
	  }	   
       }
   }
   return EXIT_SUCCESS;
}

int   matl_Zalpha_product(const MatL *matl, const double alpha,MatL *res) 
{
   int l,i,j;
   double co,si,x,y;
   extern double cos(double);
   extern double sin(double);
   
   for (l=0;l<=matl->lmax;l++)
      for (i=0;i<=2*l;i++) 
          gsl_matrix_set(res->lmat[l],i,l,gsl_matrix_get(matl->lmat[l],i,l));
	  
   for (j=1;j<=matl->lmax;j++) {
       co = cos(j*alpha);
       si = sin(j*alpha);
       for (l=j;l<=matl->lmax;l++) {
	  for (i=0;i<=2*l;i++) {
	    x = gsl_matrix_get(matl->lmat[l],i,l-j);
	    y = gsl_matrix_get(matl->lmat[l],i,l+j);
	    gsl_matrix_set(res->lmat[l],i,l-j, co*x-si*y);
	    gsl_matrix_set(res->lmat[l],i,l+j, si*x+co*y);
	  }	   
       }
   }
   return EXIT_SUCCESS;
}


/* Compute rotation matrices R_l, 0 <= l <= lmax in res given matrices J_l, 0 <= l <= lmax in matj */ 
int matl_rotation(int lmax,double alpha, double beta, double gamma,MatL *res,const MatL *matj)
{
  MatL *tmp;
  
  tmp = matl_calloc(lmax);
  
  Zalpha_matl_product(matj,beta,tmp);
  matl_matl_product(matj, tmp, res);
  Zalpha_matl_product(res,alpha,tmp);
  matl_Zalpha_product(tmp,gamma,res);
  
  matl_free(tmp);
  return EXIT_SUCCESS;
}


int   Zalpha_mat_product(int l,const gsl_matrix *mat, const double alpha,gsl_matrix *res) 
{
   int i,j;
   double co,si,x,y;
   extern double cos(double);
   extern double sin(double);

   for (j=0;j<=2*l;j++) 
       gsl_matrix_set(res,l,j,gsl_matrix_get(mat,l,j));
        
   for (i=1;i<=l;i++) {
       co = cos(i*alpha);
       si = sin(i*alpha);
	  for (j=0;j<=2*l;j++) {
	    x = gsl_matrix_get(mat,l-i,j);
	    y = gsl_matrix_get(mat,l+i,j);
	    gsl_matrix_set(res,l-i,j, co*x+si*y);
	    gsl_matrix_set(res,l+i,j, -si*x+co*y);
	  }	   
   }
   return EXIT_SUCCESS;
}

int mat_Zalpha_product(int l,const gsl_matrix  *mat, const double alpha,gsl_matrix  *res) 
{
   int i,j;
   double co,si,x,y;
   extern double cos(double);
   extern double sin(double);
   

   for (i=0;i<=2*l;i++) 
       gsl_matrix_set(res,i,l,gsl_matrix_get(mat,i,l));
	  
   for (j=1;j<=l;j++) {
       co = cos(j*alpha);
       si = sin(j*alpha);
	  for (i=0;i<=2*l;i++) {
	    x = gsl_matrix_get(mat,i,l-j);
	    y = gsl_matrix_get(mat,i,l+j);
	    gsl_matrix_set(res,i,l-j, co*x-si*y);
	    gsl_matrix_set(res,i,l+j, si*x+co*y);
	  }	   
   }
   return EXIT_SUCCESS;
}

/* For a given l, computes the rotation matrix R_l in res, given matrix J_l in matj */
int matrix_rotation(int l,double alpha, double beta, double gamma,gsl_matrix *res,const gsl_matrix *matj)
{
  gsl_matrix *tmp;
  
  tmp = gsl_matrix_calloc(2*l+1,2*l+1); 
  
  Zalpha_mat_product(l,matj,beta,tmp);
  gsl_blas_dgemm(CblasNoTrans,CblasNoTrans,1.0,matj,tmp,0.0,res);
  Zalpha_mat_product(l,res,alpha,tmp);
  mat_Zalpha_product(l,tmp,gamma,res);
  
  gsl_matrix_free(tmp);
  return EXIT_SUCCESS;
}

