/*************************************/
/* Fichier: rotation_vector.c        */
/* Version: 2.0                      */
/* Date:   July 2011                 */
/* Author: Didier Pinchon            */
/*************************************/
#include "rotation.h"

int   Zalpha_vecl_product(const VecL *vecl, const double alpha,VecL *res) 
{
   int l,i,j;
   double co,si,x,y;
   extern double cos(double);
   extern double sin(double);

   for (l=0;l<=vecl->lmax;l++)
          gsl_vector_set(res->lvec[l],l,gsl_vector_get(vecl->lvec[l],l));
        
   for (i=1;i<=vecl->lmax;i++) {
       co = cos(i*alpha);
       si = sin(i*alpha);
       for (l=i;l<=vecl->lmax;l++) {
	    x = gsl_vector_get(vecl->lvec[l],l-i);
	    y = gsl_vector_get(vecl->lvec[l],l+i);
	    gsl_vector_set(res->lvec[l],l-i, co*x+si*y);
	    gsl_vector_set(res->lvec[l],l+i, -si*x+co*y);
        }	   
   }
   return EXIT_SUCCESS;
}

int matl_vecl_product(const MatL *matl, const VecL *vecl, VecL *res)
{
  int l;
  
  if (matl->lmax!=vecl->lmax) 
     fprintf(stderr,"Dimension error in matl_vecl_product!\n");
     
  for (l=0;l<=matl->lmax;l++) {
    gsl_blas_dgemv(CblasNoTrans,1.0,matl->lmat[l],vecl->lvec[l],0.0,res->lvec[l]);
  }
  return EXIT_SUCCESS;
}
  
/* Compute rotated SH coefficients R_l V_l, 0 <= l <= lmax in res given matrices J_l, 0 <= l <= lmax in matj */ 
int vecl_rotation(int lmax,double alpha, double beta, double gamma, const VecL *x, VecL *res, const MatL *matj)
{
  VecL *tmp;
  
  tmp = vecl_calloc(lmax);
  
  Zalpha_vecl_product(x,gamma,res);
  matl_vecl_product(matj,res,tmp);
  Zalpha_vecl_product(tmp,beta,res);
  matl_vecl_product(matj,res,tmp);
  Zalpha_vecl_product(tmp,alpha,res);
  
  vecl_free(tmp);
  return EXIT_SUCCESS;
}


int   Zalpha_vec_product(int l,const gsl_vector *vec, const double alpha,gsl_vector *res) 
{
   int i,j;
   double co,si,x,y;
   extern double cos(double);
   extern double sin(double);
 
   gsl_vector_set(res,l,gsl_vector_get(vec,l));
        
   for (i=1;i<=l;i++) {
       co = cos(i*alpha);
       si = sin(i*alpha);
       x = gsl_vector_get(vec,l-i);
       y = gsl_vector_get(vec,l+i);
       gsl_vector_set(res,l-i, co*x+si*y);
       gsl_vector_set(res,l+i, -si*x+co*y);	   
   }
   return EXIT_SUCCESS;
}

/* For a given l, computes rotated SH coefficient R_l in res, given matrix J_l in matj */
int vector_rotation(int l,double alpha, double beta, double gamma,const gsl_vector *x,gsl_vector *res,const gsl_matrix *matj)
{
  gsl_vector *tmp;
  
  if (l==0) {
     gsl_vector_set(res,0, gsl_vector_get(x,0));
     return EXIT_SUCCESS;
  }
  
  tmp = gsl_vector_calloc(2*l+1); 
  
  Zalpha_vec_product(l,x,gamma,res);
  gsl_blas_dgemv(CblasNoTrans,1.0,matj,res,0.0,tmp);
  Zalpha_vec_product(l,tmp,beta,res);
  gsl_blas_dgemv(CblasNoTrans,1.0,matj,res,0.0,tmp);
  Zalpha_vec_product(l,tmp,alpha,res);
  
  gsl_vector_free(tmp);
  return EXIT_SUCCESS;
}

