#include "rotation.h"

/* Returns a random number between 0 and 1 */
double myrand()
{
  return( ((double) rand())/((double) RAND_MAX) );
}

int main(int argc, char *argv[])
{
   int lmax,l,i,j,n, iter;
   double x,y,alpha,beta,gamma,dist, norm1,norm2;
   VecL *vecl, *res1l, *res2l, *resinvl;
   MatL *matjL, *matrL;
   gsl_vector *vecres;
   gsl_matrix *matr,*matj;
   extern double fabs(double);
   
/* input data */
   lmax = atoi(argv[1]);
   alpha = atof(argv[2]);
   beta  = atof(argv[3]);
   gamma = atof(argv[4]);

/* A set of arbitrary SH coefficients */
   vecl = vecl_calloc(lmax);
   for (l=0;l<=lmax;l++)
     for (i=0;i<2*l+1;i++)
        gsl_vector_set(vecl->lvec[l],i,myrand());
     
   res1l = vecl_calloc(lmax);
   res2l = vecl_calloc(lmax);
   resinvl = vecl_calloc(lmax);
   
/* Recovering J_l matrices for l=1..lmax */
   matjL = read_lmatJ(lmax);

/* 1. Computing all rotation matrices for l=1..lmax using matl_rotation() */
   matrL = matl_calloc(lmax);
   matl_rotation(lmax, alpha, beta, gamma, matrL, matjL);

   
/* 2. Computing individual rotation matrices J_l for l=2..lmax using mat_rotation() */
   for (l=0;l<=lmax;l++) {
     matj = read_matJ(l);
 
     matr = gsl_matrix_calloc(2*l+1,2*l+1);
     matrix_rotation(l, alpha, beta, gamma, matr, matj);
   
    /* Comparing with the result previously obtained with matl_rotation() */
       for (i=0;i<=2*l;i++) {
         for (j=0;j<=2*l;j++) {
	    x = gsl_matrix_get(matrL->lmat[l],i,j);
	    y = gsl_matrix_get(matr,i,j);
	    if (x!=y){
	       printf("l= %d, x= %20.16e, y= %20.16e\n",l,x,y);
	       fprintf(stderr, "Not equality for l = %d\n",l); 
	       return EXIT_FAILURE;
	     }
	      
	 }
       }
       gsl_matrix_free(matj);
       gsl_matrix_free(matr);
     }
     printf("\n1 -> Global rotation in MatL and rotations by bands in gsl_matrices are equivalent\n");
     
/* 3-a Rotation matrix matrL applied to vecl -> res1l*/
    matl_vecl_product(matrL, vecl, res1l); 

/* 3-b Direct computation of the rotated vectors in res2l and comparison */
    vecl_rotation(lmax, alpha, beta, gamma, vecl, res2l, matjL);
    dist = 0.0;
    for (l=0;l<=lmax;l++) {
       dist = 0.0;
       for (i=0;i<2*l+1;i++) {
          x = fabs(gsl_vector_get(res1l->lvec[l],i) - gsl_vector_get(res2l->lvec[l],i));
          if (x>dist) dist=x;
       }
      
    }
    printf("2 -> Rotating all SH coefficients for lmax = %d by two methods gives equivalent results, dist = %20.16e\n",lmax,dist);
    
/* 4- Rotation by band */
    dist = 0.0;
    for (l=0;l<=lmax;l++) {
       vecres = gsl_vector_calloc(2*l+1);
       vector_rotation(l, alpha, beta, gamma, vecl->lvec[l], vecres, matjL->lmat[l]);
       for (i=0;i<2*l+1;i++) {
          x = fabs(gsl_vector_get(res2l->lvec[l],i) - gsl_vector_get(vecres,i));
          if (x>dist) dist=x;
       }
       gsl_vector_free(vecres);
    }
    printf("3 -> Rotating SH coefficients for each l=0..%d  gives identical results, dist = %20.16e\n",lmax,dist);

/* The l2-norm is invariant by rotation */
    norm1 = 0.0;
    norm2 = 0.0;
    for (l=0;l<=lmax;l++) 
       for (i=0;i<2*l+1;i++) {
          x = gsl_vector_get(vecl->lvec[l],i);
          y = gsl_vector_get(res2l->lvec[l],i);
          norm1 += x*x;
          norm2 += y*y;
       }
    printf("4 -> Conservation of the squared l2-norm by rotation : %20.16e, %20.16e -> err rel = %20.16e\n",norm1,norm2,(norm1-norm2)/norm1);   

/* Inverse rotation */
    vecl_rotation(lmax, -gamma, -beta, -alpha, res2l, resinvl, matjL);
    dist = 0.0;
    for (l=0;l<=lmax;l++) 
       for (i=0;i<2*l+1;i++) {
          x = fabs(gsl_vector_get(resinvl->lvec[l],i) - gsl_vector_get(vecl->lvec[l],i) );
          if (x>dist) dist=x;
       }   
    printf("5 -> Inverse rotation applied to the rotated vector : dist = %20.16e\n",dist);
  
    vecl_free(vecl);
    vecl_free(res1l);
    vecl_free(res2l);
    vecl_free(resinvl);
    matl_free(matjL);
    matl_free(matrL);
      
    return EXIT_SUCCESS;
}
