#include "rotation.h"
#include <time.h>

/* Returns a random number between 0 and pi */
double myrand()
{
  return( M_PI*((double) rand())/((double) RAND_MAX) );
}

double myrand01()
{
  return( ((double) rand())/((double) RAND_MAX) );
}

static clock_t cputime;

void cpu_time_reset(void)
{
  cputime = clock();
}

double cpu_time_get(void)
{
   double dt;
   dt = (double) (clock() - cputime);
   dt = dt/CLOCKS_PER_SEC;
   return dt;
}

int main(int argc, char *argv[])
{
   int lmax, itermax, i,j,l,iter;
   double x,y,*alpha,*beta,*gamma;
   double tot1,tot2, tim1, tim2, dist;
   VecL *vecl, *resl, *resl2;
   MatL *matjL, *matrL;
   
   gsl_matrix *matr;
   FILE *fres;
   extern double fabs(double);

   lmax    = atoi(argv[1]);
   itermax = atoi(argv[2]);

   if ((fres = fopen("res.dat","w")) == NULL) {
    fprintf(stderr,"Cannot open output file res.out for writing!.\n");
    return EXIT_FAILURE;
   }

   
/* First read J_l matrices for 0 <=l <= lmax. Not included in time statistics */
   matjL = read_lmatJ(lmax);
   matrL = matl_calloc(lmax);

/* A set random SH coefficients. Not included in time statistics */
   vecl = vecl_calloc(lmax);
   for (l=0;l<=lmax;l++)
     for (i=0;i<2*l+1;i++)
        gsl_vector_set(vecl->lvec[l],i,myrand01());
        
   resl = vecl_calloc(lmax);     
   resl2 = vecl_calloc(lmax);
/* Vectors of random angles alpha, beta, gamma. Not included in time statistics */
   alpha = (double *)calloc(itermax,sizeof(double));
   beta  = (double *)calloc(itermax,sizeof(double));
   gamma = (double *)calloc(itermax,sizeof(double));
   for (iter=0;iter<itermax;iter++) {
     alpha[iter] = 2*myrand();
     beta[iter] = myrand();
     gamma[iter] = 2*myrand();
   }
   
   tot1 = 0.0;
   tot2 = 0.0;
   
   for (l=2;l<=lmax;l++) {
     /* first method: computing the rotation matrix and then applying it to the vector of SH coefficients */
     cpu_time_reset();
     for (iter=0;iter<itermax;iter++) {
        matrix_rotation(l, alpha[iter], beta[iter], gamma[iter], matrL->lmat[l], matjL->lmat[l]);
        gsl_blas_dgemv(CblasNoTrans,1.0,matrL->lmat[l],vecl->lvec[l],0.0,resl->lvec[l]);
     }		
     tim1 = cpu_time_get()/itermax;
     tot1 += tim1;

     /* second method: computing directly the rotated vectors */
     cpu_time_reset();
     for (iter=0;iter<itermax;iter++) {
       vector_rotation(l, alpha[iter], beta[iter], gamma[iter], vecl->lvec[l], resl->lvec[l], matjL->lmat[l]);
     }
     tim2 = cpu_time_get()/itermax;
     tot2 += tim2;
     
     /* Computing again by the two methods without recording execution time to check that the results are equivalent */
     dist = 0.0;
     for (iter=0;iter<itermax;iter++) {
        matrix_rotation(l, alpha[iter], beta[iter], gamma[iter], matrL->lmat[l], matjL->lmat[l]);
        gsl_blas_dgemv(CblasNoTrans,1.0,matrL->lmat[l],vecl->lvec[l],0.0,resl->lvec[l]);
        vector_rotation(l, alpha[iter], beta[iter], gamma[iter], vecl->lvec[l], resl2->lvec[l], matjL->lmat[l]);
        for (i=0;i<2*l+1;i++) {
          dist = fabs(gsl_vector_get(resl->lvec[l],i) - gsl_vector_get(resl2->lvec[l],i));
          if (x>dist) dist = x;
	}
     }	
     
     printf("%d %20.16e %20.16e %20.16e %20.16e %20.16e\n",l,tim1,tim2,tot1,tot2,dist);
     fprintf(fres,"%d %20.16e %20.16e %20.16e %20.16e %20.16e\n",l,tim1,tim2,tot1,tot2,dist);
   }
   
   fclose(fres);
   free(alpha);
   free(beta);
   free(gamma);
   vecl_free(vecl);
   vecl_free(resl);
   vecl_free(resl2);
   matl_free(matjL);
   matl_free(matrL);
  
   return EXIT_SUCCESS;
}
