/// @file
////////////////////////////////////////////////////////////////////////////////
///
/// Copyright (C) 2009      University of Toronto
///
////////////////////////////////////////////////////////////////////////////////
///
///  module     : condition number optimization
///
///  author     : lessig@dgp.toronto.edu
///
///  project    : Spherical Harmonics rotation
///
///  description: Interface for optimizer
///
///  license    : See README for details.
///
////////////////////////////////////////////////////////////////////////////////

#ifndef _OPTIMIZER_H_
#define _OPTIMIZER_H_

// includes, file
#include <config.h>

// includes, project
#include <BFGSwrap.h>
#include <energy.h>

class Optimizer {

public:

  //! Constructor
  Optimizer();

  //! Destructor
  ~Optimizer();

public:

  //! Initialize optimize
  //! @param fname_lambdas  file name for the initial points of the optimization
  //! @param opt_time  maximum optimization time (in seconds)
  void  init( const std::string& fname_in, const unsigned int opt_time);
 
  //! Run the optimizer 
  void  run();

  //! Cleanup and write result
  //! @param fname_lambdas  file name for optimized points 
  void  finish( const std::string& fname_lambdas);

private:

  // The optimization is unconstrained but it is desirable that the final points
  // are probably limited to [0,\pi] x [0,2\pi]
  void fixDomainLambda();

private:

  // data

  //! sampling points
  LambdaVec  lambdas;

  //! Energy computation
  Energy  energy_comp;

  //! Current energy
  double  energy;

  //! Current gradient
  Vec  gradient;

  //! max optimization time (in sec)
  unsigned int opt_time_max;

private:

  //! Copy constructor
  //! @note Not implemented
  Optimizer( const Optimizer&);  

  //! Assignment operator
  //! @note Not implemented
  Optimizer& operator=( const Optimizer&);  

};

#endif // _OPTIMIZER_H_

