// L-BFGS wrapper code, written by Denis Zorin

#include "BFGSwrap.h"
#include <string.h>

// Fortran call prototypes
extern "C" { 
  // Fortran string copy behavior is different 
  // from strncpy()
  // The last two args are string lengths
  int s_copy(char*, char*,int,int); 

  // optimization function
  int setulb_( int* n, int* m,
     double* x, 
     double* l, double* u, int* nbd,
     double* f, double* g,
     double* factr, double* pgtol,
     double* wa,  int* iwa,
     char* task,  int* iprint,
     char* csave, int* lsave, int* isave, double* dsave, 
     int tasklen, int csavelen);
}

BFGSoptimizer::BFGSoptimizer( int n, double* x0, Domain* ds, int m):
  dim(n), 
  matdim(m),
  mydomain(false), 
  domain(ds), 
  factr( 1),
  pgtol( 10e-10),
  iprint(-1)
{     
  s_copy(task, "START", 60, 5); 
  x = x0;
  wa = new double[2*m*n+4*n+12*m*m+12*m];
  iwa = new int[3*n];
  if(domain == 0) {
    domain = new Domain(n);
    mydomain =true;
  } else
    assert( domain->size() == n );
}


BFGSoptimizer::Results BFGSoptimizer::nextIteration(double f, double* grad ) {   
  double zero = 0.0;

  setulb_(&dim,&matdim,
             x,domain->lower(),domain->upper(),domain->boundtype(),
             &f,grad,
             &zero,&zero,
             wa,iwa,
             task,&iprint,
             csave,lsave,isave,dsave, 60, 60);   
  if( !strncmp(task, "FG",2)) return EvaluationRequest;
  if( !strncmp(task, "NEW_X",5)) return IterationCompleted;
  if( !strncmp(task, "CONV",4)) return  Converged;
  if( !strncmp(task, "ABNO",4)) return  Abnormal;
  if( !strncmp(task, "ERROR",4)) return  DataError;
  else return Unrecognized;
}




