/// @file
////////////////////////////////////////////////////////////////////////////////
///
/// Copyright (C) 2009      University of Toronto
///
////////////////////////////////////////////////////////////////////////////////
///
///  module     : main
///
///  author     : {lessig|tyler}@dgp.toronto.edu
///
///  project    : Spherical Harmonics rotation
///
///  description: Command line argument handling
///
///  license    : See README for details.
///
////////////////////////////////////////////////////////////////////////////////

#ifndef CMD_LINE_ARGS_H
#define CMD_LINE_ARGS_H

// includes, project
#include <cutil.h>

////////////////////////////////////////////////////////////////////////////////
//! Get input argument
////////////////////////////////////////////////////////////////////////////////
void
getCmdLineArgMaxBand( int argc, char** argv, unsigned int& max_band) {

  // extract the command line argument
  int temp = -1;
  cutGetCmdLineArgumenti( argc, (const char**) argv, "max-band", &temp);

  // check if valid command line argument was provided
  if( temp > 0) {
    max_band = (unsigned int) temp;
  }

  printf( "Maximum band : %i\n", max_band);
}

////////////////////////////////////////////////////////////////////////////////
//! Get input argument
////////////////////////////////////////////////////////////////////////////////
void
getCmdLineArgNumRotations( int argc, char** argv, unsigned int& num_rotations) {

  // extract the command line argument
  int temp = -1;
  cutGetCmdLineArgumenti( argc, (const char**) argv, "num-rotations", &temp);

  // check if valid command line argument was provided
  if( temp > 0) {
    num_rotations = (unsigned int) temp;
  }

  printf( "Number of rotations : %i\n", num_rotations);
}

////////////////////////////////////////////////////////////////////////////////
//! Get input argument
////////////////////////////////////////////////////////////////////////////////
void
getCmdLineArgNumItersTiming(int argc,char** argv, unsigned int& num_iterations) {

  // extract the command line argument
  int temp = -1;
  cutGetCmdLineArgumenti( argc, (const char**) argv, "num-iterations", &temp);

  // check if valid command line argument was provided
  if( temp > 0) {
    num_iterations = (unsigned int) temp;
  }

  printf( "Number of iterations for timing : %i\n", num_iterations);
}


////////////////////////////////////////////////////////////////////////////////
//! Get input argument
////////////////////////////////////////////////////////////////////////////////
void
getCmdLineArgAccuracy(int argc,char** argv, int& perform_accuracy) {

  // extract the command line argument
  int temp = -1;
  perform_accuracy=0;
  cutGetCmdLineArgumenti( argc, (const char**) argv, "accuracy", &temp);

  // check if valid command line argument was provided
  if( temp > -1) {
    perform_accuracy=1;
  }
}

#endif // CMD_LINE_ARGS_H
