c     ---------------------------
      subroutine rotmat(A,l,T,st)
c     ---------------------------
      implicit double precision(a-h,o-z)
      dimension T(9), A((l*(l+1)*(l+2)*4/3)+(l+1))
      dimension st(2*l)
c
c     Calculates all rotation matrices for real spherical harmonics of
c     order 0 to l.  These matrices are returned in A in order of
c     increasing l.  For example, the first element in A is 1 which
c     is the rotation matrix for s-type harmonics.  The next nine
c     elements make up the 3x3 matrix for the p-type harmonics and
c     the next 25 elements make up the 5x5 matrix for the d-type harmonics
c     and etc.  The group of elements in A, starting from position
c     start=(l-1)*l*(l+1)*4/3 + (l+1), which make up the (2l+1)x(2l+1)
c     matrix, R, may be unpacked in the following manner :
c     R(i,j) = A(start+((j-1)*(2l+1))+i-1).  The labelling of the columns
c     and rows of R with respect to ml is given in the subroutine cormat 
c     below.
c     The array st is a scratch array of length 2l.   The elements of
c     this array will be defined as follows, st(i) = sqrt(i).  The
c     reason for the storage of these square roots, which are required 
c     in cormat, is to save time by not having to recalculate many times
c     the square roots of integers required in cormat.
c     Code by J. Ivanic October 1996.
c
c     Comments (CL): s-type, p-type etc. is a labelling of the bands 
c                    common in chemistry and related areas.
c
      do 20 i=1,2*l
	 st(i) = sqrt(i*1.d0)
   20 continue
c
      A(2) = T(9)
      A(3) = T(7)
      A(4) = T(8)
      A(5) = T(3)
      A(6) = T(1)
      A(7) = T(2)
      A(8) = T(6)
      A(9) = T(4)
      A(10) = T(5)
c
      do 30 ll=1,l-1
	 iR1t = 2
	 iRLs = ((ll-1)*ll*(ll+1)*4/3) + ll + 1
	 iRUs = (ll*(ll+1)*(ll+2)*4/3) + ll + 2
         call cormat(A(iRLs),A(iRUs),ll,A(iR1t),st)
   30 continue
c
      return
      end
c
c     -------------------------------
      subroutine cormat(RL,RU,l,R,sq)
c     -------------------------------
      implicit double precision(a-h,o-z)
      dimension RL(2*l+1,2*l+1), RU(2*l+3,2*l+3), R(3,3), sq(*)
c
c     Calculates the rotation matrix, RU (RUpper), for the real spherical harmonics
c     of order l+1 from the rotation matrix for the harmonics of order l, RL,
c     and the 3x3 rotation matrix R for l = 1.
c     (Note that l=letter "el" and 1 = number "one").   
c     Relation between the rows(i) and columns(j) of the matrices and m is as follows:
c         	i,j = 1              equivalent to m = 0
c    	        i,j = 2,4,6,...      equivalent to m = 1,2,3,..   
c               i,j = 3,5,7,....     equivalent to m = -1,-2,-3,...
c     Corrot(= calculate one real rotation matrix) uses a recurrence 
c     procedure expressing the elements of R(l+1) in terms of those of R(l) and
c     R(=rotation matrix for p functions).  R is obtained from T by appropriate
c     permutations of elements.
c     The array sq(i) contains the square roots of the integers i=1,2*l+2.  
c     These have been evaluated in subroutine rotmat and are included only to
c     speed up the program.
c     Algorithm by J. Ivanic and K. Ruedenberg, J. Phys, Chem, 100, 6342-6347 (1996)
c     Code by J. Ivanic, October 1995
c
c     First row of RU, ie m=0 
c
      i=1
c
      do 100 j=1,(2*l+1)
         m2 = j/2
c
	 RU(i,j) = FORM1(R,RL,1,l,j,m2,sq)
c
  100 continue
c
      j=2*l+2
      m2=l+1
      RU(i,j) = FORM1(R,RL,2,l,j,m2,sq)
c
      j=2*l+3
      RU(i,j) = FORM1(R,RL,3,l,j,m2,sq) 
c
c     First row finished, now for rows corresponding to m = 1, 2, 3,...,(2l+3)
c
      do 300 i=2,2*l+2,2
	 m1=i/2
	 do 200 j=1,(2*l+1)
            m2 = j/2
c
	    RU(i,j) = FORM2(R,RL,1,l,i,j,m1,m2,sq)
c
  200    continue
c
	 j=2*l+2
	 m2=l+1
	 RU(i,j) = FORM2(R,RL,2,l,i,j,m1,m2,sq)
c
	 j=2*l+3
	 RU(i,j) = FORM2(R,RL,3,l,i,j,m1,m2,sq)
c
  300 continue
c
c     Remaining rows, ie those corresponding to m = -1, -2, -3,...,-(2l+3)
c
      do 500 i=3,2*l+3,2
	 m1=(i-1)/2
	 do 400 j=1,(2*l+1)
            m2 = j/2
c
	    RU(i,j) = FORM3(R,RL,1,l,i,j,m1,m2,sq)
c
  400    continue
c
	 j=2*l+2
	 m2=l+1
	 RU(i,j) = FORM3(R,RL,2,l,i,j,m1,m2,sq)
c
	 j=2*l+3
	 RU(i,j) = FORM3(R,RL,3,l,i,j,m1,m2,sq)
c
  500 continue 
c
      return
      end
c
c     **********************************************
      double precision function FORM1(R,RL,ii,l,j,m,sq)
c     **********************************************
c
      implicit double precision(a-h,o-z)
      dimension R(3,3), RL(2*l+1,2*l+1),sq(*)
      parameter (zero=0.0d+00,one=1.0d+00)
      parameter (RT2 = 1.4142 13562 37309 50488d0)
    
c
      s = (l+1)*1.d0
      q = m*1.d0
      t1 = s + q
      t2 = s - q
c
      if (ii.eq.1) then
         A1 = s/(sq(t1)*sq(t2))
         A2 = ((sq(s)*sq(s-one))/(sq(t1)*sq(t2)))/RT2
      else
	 A1 = s/(sq(t1)*sq(t1-one))
	 A2 = ((sq(s)*sq(s-one))/(sq(t1-one)*sq(t1)))/RT2
      endif
c
      P1 = CalcP(ii,1,1,j,RL,R,l)
      P2 = CalcP(ii,2,2,j,RL,R,l)
      P3 = CalcP(ii,3,3,j,RL,R,l)
c
c
      FORM1 = A1*P1 - A2*(P2 + P3)
c
      return
      end
c
c     ****************************************************
      double precision function FORM2(R,RL,ii,l,i,j,m1,m2,sq)
c     ****************************************************
c
      implicit double precision(a-h,o-z)
      dimension R(3,3), RL(2*l+1,2*l+1),sq(*)
      parameter (zero=0.0d+00, one=1.0d+00, two=2.0d+00)
      parameter (RT2 = 1.4142 13562 37309 50488d0)
c
      s=(l+1)*1.d0
      q1=m1*1.d0
      q2=m2*1.d0
      t1 = s+q1
      t2 = s+q2
      t3 = s-q1
      t4 = s-q2
c
      if (ii.eq.1) then
         A1 = (sq(t1)*sq(t3))/(sq(t2)*sq(t4))
         A2 = ((sq(t1)*sq(t1-one))/(sq(t2)*sq(t4)))/two
         A3 = ((sq(t3)*sq(t3-one))/(sq(t2)*sq(t4)))/two
      else
	 A1 = (sq(t1)*sq(t3))/(sq(t2)*sq(t2-one))
	 A2 = ((sq(t1)*sq(t1-one))/(sq(t2)*sq(t2-one)))/two
	 A3 = ((sq(t3)*sq(t3-one))/(sq(t2)*sq(t2-one)))/two
      endif
c
      P1 = CalcP(ii,1,i,j,RL,R,l)
c
      if (i-2.lt.1) then
	 k=1
      else 
	 k=i-2
      endif
c
      P2 = CalcP(ii,2,k,j,RL,R,l)
      P3 = CalcP(ii,3,i-1,j,RL,R,l)
      P4 = CalcP(ii,2,i+2,j,RL,R,l)
      P5 = CalcP(ii,3,i+3,j,RL,R,l)
c
      if (m1.eq.one) then
	 C1 = RT2
	 C2 = zero
      else
	 C1 = one
	 C2 = one
      endif
c
      FORM2 = A1*P1 + A2*(C1*P2 - C2*P3) - A3*(P4 + P5)
c
      return
      end
c
c     ****************************************************
      double precision function FORM3(R,RL,ii,l,i,j,m1,m2,sq)
c     ****************************************************
c
      implicit double precision(a-h,o-z)
      dimension R(3,3), RL(2*l+1,2*l+1),sq(*)
      parameter (zero=0.0d+00, one=1.0d+00, two=2.0d+00)
      parameter (RT2 = 1.4142 13562 37309 50488d0)
c
      s=(l+1)*1.d0
      q1=m1*1.d0
      q2=m2*1.d0
      t1 = s+q1
      t2 = s+q2
      t3 = s-q1
      t4 = s-q2
c
      if (ii.eq.1) then
         A1 = (sq(t1)*sq(t3))/(sq(t2)*sq(t4))
         A2 = ((sq(t1)*sq(t1-one))/(sq(t2)*sq(t4)))/two
         A3 = ((sq(t3)*sq(t3-one))/(sq(t2)*sq(t4)))/two
      else
	 A1 = (sq(t1)*sq(t3))/(sq(t2)*sq(t2-one))
	 A2 = ((sq(t1)*sq(t1-one))/(sq(t2)*sq(t2-one)))/two
	 A3 = ((sq(t3)*sq(t3-one))/(sq(t2)*sq(t2-one)))/two
      endif
c
      P1 = CalcP(ii,1,i,j,RL,R,l)
      if (i-3.lt.1) then
	 k=1
      else 
	 k=i-3
      endif
      P2 = CalcP(ii,3,k,j,RL,R,l)
      P3 = CalcP(ii,2,i-2,j,RL,R,l)
      P4 = CalcP(ii,2,i+2,j,RL,R,l)
      P5 = CalcP(ii,3,i+1,j,RL,R,l)
c
      if (m1.eq.one) then
	 C1 = RT2
	 C2 = zero
      else
	 C1 = one
	 C2 = one
      endif
c
      FORM3 = A1*P1 + A2*(C1*P2 + C2*P3) - A3*(P4-P5)
c
      return
      end
c
c     ****************************************************
      double precision function CalcP(ii,m1,m2,m3,RL,R,l)
c     ****************************************************
c
      implicit double precision(a-h,o-z)
      dimension RL(2*l+1,2*l+1), R(3,3)
c
      if (ii.eq.1) CalcP = R(m1,1)*RL(m2,m3)
c
      if (ii.eq.2) CalcP = R(m1,2)*RL(m2,m3-2) - R(m1,3)*RL(m2,m3-1)
c
      if (ii.eq.3) CalcP = R(m1,3)*RL(m2,m3-3) + R(m1,2)*RL(m2,m3-2)
c
      return
      end
c
