# Gaunt coefficients

# triangle functions
istriangle := proc(l1,l2,l3)
   evalb(l1<=l2+l3 and l1 >= abs(l2-l3) and
         l2<=l1+l3 and l2 >=abs(l1-l3) and
         l3<=l1+l2 and l3 >= abs(l1-l2)
         );
end:

Delta := proc(j1,j2,j3)
  sqrt((j1+j2-j3)!*(j1-j2+j3)!*(-j1+j2+j3)!/(j1+j2+j3+1)!);
end:

# completely explicit definition
gaunt := proc(l1,l2,l3,m1,m2,m3)
   local L,kmin,kmax,k,res,sss;
   if not(m1+m2+m3=0) then RETURN(0); fi;
   if type(l1+l2+l3,odd) then RETURN(0); fi;
   L := iquo(l1+l2+l3,2);
   kmin := max(-l3+l1+m2,-l3+l2-m1,0);
   kmax := min(l2+m2,l1-m1,l1+l2-l3);
   sss := 0;
   for k from kmin to kmax do
     sss := sss + (-1)^k/(k!*(k+l3-l1-m2)!*(k+l3-l2+m1)!*(k+l3-l2+m1)!*(l1-m1-k)!*(l2+m2-k)!);
   od;
   res := Delta(l1,l2,l3)^2*(-1)^(L+l3+m1-m2)*sqrt((2*l1+1)*(2*l2+1)*(2*l3+1))*
     L!/((L-l1)!*(L-l2)!*(L-l3)!)*
     sqrt((l1+m1)!*(l1-m1)!*(l2+m2)!*(l2-m2)!*(l3+m3)!*(l3-m3)!)*sss;
   simplify(res);
end:

# mathematical definition
defGaunt := proc(l1,l2,l3,m1,m2,m3)
  simplify(2*sqrt(Pi)*intSpherik(Y(l1,m1)*Y(l2,m2)*Y(l3,m3)));
end:

# definition with Wigner' 3j symbols
wigner3j := proc(j1,j2,j3,m1,m2,m3)
  local sss,k,kmin,kmax,res,den;
  if not(istriangle(j1,j2,j3)) or not(m1+m2+m3=0) then RETURN(0); fi;
  kmax := min(j1+j2-j3,j1-m1,j2+m2);
  kmin := max(0,j1-j3+m2,j2-j3-m1);
  sss := 0;
  for k from kmin to kmax do
    den := k!*(j1+j2-j3-k)!*(j1-m1-k)!*(j2+m2-k)!*(j3-j2+m1+k)!*(j3-j1-m2+k)!;
    sss := sss + (-1)^k/den;
  od;
  res := (-1)^(j1-j2-m3)*sqrt(triangle(j1,j2,j3))*sqrt((j1+m1)!*(j1-m1)!*(j2+m2)!*(j2-m2)!*(j3+m3)!*(j3-m3)!)*sss;
simplify(res);
end:

wigner3j0 := proc(j1,j2,j3)
   local sj,g,res;
   sj := j1+j2+j3;
   if type(sj,odd) then RETURN(0); fi;
   g := iquo(sj,2);
   res := (-1)^g*sqrt((2*(g-j1))!*(2*(g-j2))!*(2*(g-j3))!/(2*g+1)!)*g!/((g-j1)!*(g-j2)!*(g-j3)!);
   simplify(res);
end:

gauntY := proc(l1,l2,l3,m1,m2,m3)
  simplify( sqrt((2*l1+1)*(2*l2+1)*(2*l3+1)/(4*Pi))*wigner3j0(l1,l2,l3)*wigner3j(l1,l2,l3,m1,m2,m3) );
end:

# real Gaunt coefficients
# mathematical definition
realGaunt := proc(l1,l2,l3,m1,m2,m3)
   intSpherik(SS(l1,m1)*SS(l2,m2)*SS(l3,m3));
end:

