# mkMatJ.mpl
# Storing J_l by blocks in exactMatJ.dat

read("rotation.mpl");

expo2 := proc(n)
  local lf;
  lf := ifactors(n);
  if not(lf[1]=1) then RETURN("1:not a power of 2"); fi;
  if nops(lf[2])=0 then RETURN(0); fi;
  if not(nops(lf[2])=1 and lf[2,1,1]=2) then RETURN("2:not a power of 2"); fi;
  lf[2,1,2];
end:
  
decompPureQuadratic := proc(x)
  local lf,f,den,a,b, check,pow2;
  if x=0 then RETURN([0,1,0]); fi;
  if evalf(x)<0 then a:=-1; else a:=+1; fi;
  den := denom(x);
  pow2 := expo2(den);
  lf := ifactors(numer(x)^2);
  b := 1;
  for f in lf[2] do
    a := a*f[1]^iquo(f[2],2);
    b := b*f[1]^irem(f[2],2);
  od;
  check := simplify(x - a*sqrt(b)/2^pow2);
  if not(check=0) then ERROR("error in decompGaunt!"); fi;
  [a,pow2,b];
end:

fprintfSymMat := proc(f,mat)
   local i,j,dec;
   for i to rowdim(mat) do
     for j to i do
        dec := decompPureQuadratic(mat[i,j]);
        fprintf(f,"%d %d %d\n",op(dec));
     od;
   od;
end:

fprintfGenMat := proc(f,mat)
   local i,j,dec;
   for i to rowdim(mat) do
     for j to coldim(mat) do
        dec := decompPureQuadratic(mat[i,j]);
        fprintf(f,"%d %d %d\n",op(dec));
     od;
   od;
end:

mkMatJFile := proc (fname,Lmin,Lmax)
 local f,l,k,r;
 f := fopen(fname,APPEND);
 for l from Lmin to Lmax do
    fprintf(f,"###l= %d\n",l);
    k := iquo(l,2,'r');
    if r=0 then
       fprintfSymMat(f,mkAyz(k));
       fprintfGenMat(f,mkAxz(k));
       fprintfSymMat(f,mkA1(k+1));
    else
       fprintfSymMat(f,mkAxyz(k));
       fprintfGenMat(f,mkAz(k+1));
       fprintfSymMat(f,mkAx(k+1));
    fi; 
    print(l);
 od:
 fclose(f);
end:

#x:=5:for i from x to x do mkGaunt(cat("gaunt",x,".dat"),i,i): od;

#mkMatJFile("exactMatJ.dat",2,5);