#include "gaunt.h"

Gaunt_Coeffs *init_gaunt_coeffs1(int lmax)
{
   int n,nbm2,l1,l2,l3,m3,im2;
   double x;
   Gaunt_Coeffs *ga;
   FILE *fin;
   
   if ((fin = fopen("numgaunt1.dat","r")) == NULL) {
     fprintf(stderr,"Cannot open input file numgaunt.dat.\n");
   }
   
   ga = (Gaunt_Coeffs *)calloc(1,sizeof(Gaunt_Coeffs));
   ga->lmax = lmax;
   ga->len = fN1(lmax);
   ga->gaunt = (double **)calloc(ga->len, sizeof(double *));
   
   n = -1;
   for (l1=0;l1<=lmax;l1++) {
     for (l2=0;l2<=l1;l2++) {
       for (l3=0;l3<=l2;l3++) {
	  for (m3=0;m3<=l3;m3++) {
	       n+=1;
	       if (istriangle(l1,l2,l3)==0 && (l1+l2+l3)%2==0) {
	          nbm2 = l2+imin(l2,l1-m3)+1;
	          ga->gaunt[n] = (double *)calloc(nbm2 ,sizeof(double));      
	          for (im2=0;im2<nbm2;im2++) {
	            fscanf(fin,"%lf",&x);
		    ga->gaunt[n][im2] = x;
	          }	  
	       }  
	  }
       }
     }
   }

  fclose(fin);
  return ga;
}

void free_Gaunt_Coeffs(Gaunt_Coeffs *GAUNT)
{
  int n;
  for (n=0;n<GAUNT->len;n++)
     free(GAUNT->gaunt[n]);
  free(GAUNT->gaunt);
  free(GAUNT);   
  return;
}

double gaunt1(int l1, int l2, int l3, int m1, int m2, int m3, Gaunt_Coeffs *GAUNT)
{
   int il1,il2,il3,im1,im2,im3;
   unsigned long n,m;
   /* nullity cases */
   if (l1 >l2+l3 || l2>l1+l3  || l3>l1+l2  || (l1+l2+l3)%2!=0 || m1+m2+m3!=0 || abs(m1)>l1 || abs(m2)>l2 || abs(m3)>l3 )
      return 0;
   /* reduction function: permutation of (l1,m1), (l2,m2), (l3,m3) to get il1<=il2<=il3 */
   if (l1<l2) {
      if (l2<l3) { /* l1 < l2 < l3 */
        il1 = l3; im1 = m3; il2 = l2; im2 = m2; il3 = l1; im3 = m1;
      }
      else {
         if (l1<l3) { /* l1 < l3 <= l2 */
	    il1 = l2; im1 = m2; il2 = l3; im2 = m3; il3 = l1; im3 = m1;
	 }
	 else { /* l3 <= l1 < l2 */
            il1 = l2; im1 = m2; il2 = l1; im2 = m1; il3 = l3; im3 = m3;
	 }
      }
   }
   else {
      if (l2<l3) {
          if (l1<l3) { /* l2 <= l1 < l3 */
	     il1 = l3; im1 = m3; il2 = l1; im2 = m1; il3 = l2; im3 = m2;
	  }
	  else {  /* l2 < l3 <= l1 */
	     il1 = l1; im1 = m1; il2 = l3; im2 = m3; il3 = l2; im3 = m2;
	  }
      }
      else { /* l3 <= l2 <= l1 */
         il1 = l1; im1 = m1; il2 = l2; im2 = m2; il3 = l3; im3 = m3;
     }
   }
   /* symmetry on (im1,im2,im3) to get im3 >=0 */
   if (im3<0) {
     im1 = -im1;
     im2 = -im2;
     im3 = -im3;
   }  
   /* Rasch & Yu index function (il1,il2,il3,im3) */   
   n = il1;
   n = (n*(6 + n*(11 + n*(6+n))))/24;
   m = il2;
   m = (m*(2 + m*(3 + m)))/6;
   n = n + m + (il3*(il3+1))/2 + im3;
   
   return GAUNT->gaunt[n][il2+im2];  
}
