#include "gaunt.h"

Gaunt_Coeffs *init_gaunt_coeffs2(int lmax)
{
   int n,nbm2,l1,l2,l3,m3,im2;
   double x;
   Gaunt_Coeffs *ga;
   FILE *fin;
   
   if ((fin = fopen("numgaunt2.dat","r")) == NULL) {
     fprintf(stderr,"Cannot open input file numgaunt2.dat.\n");
   }
   
   ga = (Gaunt_Coeffs *)calloc(1,sizeof(Gaunt_Coeffs));
   ga->lmax = lmax;
   ga->len = fN2(lmax);
   ga->gaunt = (double **)calloc(ga->len, sizeof(double *));
   
   n = -1;
   for (l1=0;l1<=lmax;l1++) {
     for (l2=iceil(l1,2);l2<=l1;l2++) {
       for (l3=l1-l2;l3<=l2;l3++) {
          if ((l1+l2+l3)%2==0) {
	     for (m3=0;m3<=l3;m3++) {
	          n+=1;       
		  if (l1==l2) {
		     nbm2 = l2 - iceil(m3,2) + 1;
		  }
		  else if (m3==0) {
		     nbm2 = l2 + 1;
		  }   
		  else if (l2==l3) {
		     nbm2 = l2 + imin(l1-m3,m3)+1;
		  }   
		  else {
		     nbm2 = l2+imin(l2,l1-m3)+1;         
		  }      
	          ga->gaunt[n] = (double *)calloc(nbm2 ,sizeof(double));      
	          for (im2=0;im2<nbm2;im2++) {
	            fscanf(fin,"%lf",&x);
		    ga->gaunt[n][im2] = x;
	          }	  
	       }  
	  }
       }
     }
   }

  fclose(fin);
  return ga;
}

double gaunt2(int l1, int l2, int l3, int m1, int m2, int m3, Gaunt_Coeffs *GAUNT)
{
   int il1,il2,il3,im1,im2,im3,ex,u,v,n;
   unsigned long ind;
   /* nullity cases */
  /* if (istriangle(l1,l2,l3)!=0 || (l1+l2+l3)%2!=0 || m1+m2+m3!=0 || abs(m1)>l1 || abs(m2)>l2 || abs(m3)>l3 ) */
   if (l1 >l2+l3 || l2>l1+l3  || l3>l1+l2 || (l1+l2+l3)%2!=0 || m1+m2+m3!=0 || abs(m1)>l1 || abs(m2)>l2 || abs(m3)>l3 )
      return 0;
   /* reduction function */
   /* permutation of (l1,m1), (l2,m2), (l3,m3) to get il1<=il2<=il3 */
   if (l1<l2) {
      if (l2<l3) { /* l1 < l2 < l3 */
        il1 = l3; im1 = m3; il2 = l2; im2 = m2; il3 = l1; im3 = m1;
      }
      else {
         if (l1<l3) { /* l1 < l3 <= l2 */
	    il1 = l2; im1 = m2; il2 = l3; im2 = m3; il3 = l1; im3 = m1;
	 }
	 else { /* l3 <= l1 < l2 */
            il1 = l2; im1 = m2; il2 = l1; im2 = m1; il3 = l3; im3 = m3;
	 }
      }
   }
   else {
      if (l2<l3) {
          if (l1<l3) { /* l2 <= l1 < l3 */
	     il1 = l3; im1 = m3; il2 = l1; im2 = m1; il3 = l2; im3 = m2;
	  }
	  else {  /* l2 < l3 <= l1 */
	     il1 = l1; im1 = m1; il2 = l3; im2 = m3; il3 = l2; im3 = m2;
	  }
      }
      else { /* l3 <= l2 <= l1 */
         il1 = l1; im1 = m1; il2 = l2; im2 = m2; il3 = l3; im3 = m3;
     }
   }
   /* symmetry on (im1,im2,im3) to get im3 >=0 */
   if (im3<0) {
     im1 = -im1;
     im2 = -im2;
     im3 = -im3;
   }  
   /* other symmetries */
   if (il1==il2) {
       if (im2>-iceil(im3,2)) {
          ex = im2; im2 = im1; im1 = ex;
       }	  
   }    
   else if (im3==0) {
       if (im2>0) {
          im1 = -im1; im2 = -im2;
       }
   }    
   else if (il2==il3) {
       if (im2>im3) {
          ex = im2; im2 = im3; im3 = ex;
       }
   }
   /* End of reduction function */
   /* New index function f2(il1,il2,il3,im3) */   
   u = il1/2;
   n = (il1+il2+il3)/2;
   if (il1%2==0)  {
      v = il2-u;
      ind = ( u*( ((3*u + 10)*u + 33)*u + 2))/12 + (v*(v+1))/2 + u*(v*(v+5))/2 + n*(n-3*u-v) + im3;
      }
   else {
      v = il2-u-1;
      ind = ((u+1)*( ((3*u + 13)*u + 44)*u + 24)  )/12 +(v*(v+3))/2 + (u*v*(v+5))/2 + n*(n-3*u-v-2) + im3;
   }      

   return GAUNT->gaunt[ind][il2+im2];  
}
