#include "gaunt.h"

Gaunt_Coeffs_3 *init_gaunt_coeffs3(int lmax)
{
   int n,nbm2,l1,l2,l3,m3,im2;
   double x;
   Gaunt_Coeffs_3 *ga;
   FILE *fin;
  
   ga = (Gaunt_Coeffs_3 *)calloc(1,sizeof(Gaunt_Coeffs_3));
   ga->lmax = lmax;
   ga->len0 = fN3bar(lmax);
   ga->len3 = fN3(lmax);
   ga->coef0 = (double *)calloc(ga->len0, sizeof(double));
   ga->w3j = (double **)calloc(ga->len3, sizeof(double *));
   
   if ((fin = fopen("numcoef0.dat","r")) == NULL) {
     fprintf(stderr,"Cannot open input file num.dat.\n");
   }
   n = -1;
   for (l1=0;l1<=lmax;l1++) {
     for (l2=iceil(l1,2);l2<=l1;l2++) {
       for (l3=l1-l2;l3<=l2;l3++) {
          if ((l1+l2+l3)%2==0) {
	          n+=1;       
	          fscanf(fin,"%lf",&x);	  
		    ga->coef0[n] = x; 
	  }
       }
     }
   }
   fclose(fin);  

   if ((fin = fopen("numw3j.dat","r")) == NULL) {
     fprintf(stderr,"Cannot open input file num.dat.\n");
   }
   n = -1;
   for (l1=0;l1<=lmax;l1++) {
     for (l2=iceil(l1,2);l2<=l1;l2++) {
       for (l3=l1-l2;l3<=l2;l3++) {
          if ((l1+l2+l3)%2==0) {
	     for (m3=0;m3<=l3;m3++) {
	       if ( ( (m3<l1-l2) && ((l1+l2+m3)%2==1)) || (m3>=l1-l2) ) {
	          n+=1;       
		  if (l1==l2) {
		     nbm2 = l2 - iceil(m3,2) + 1;
		  }
		  else if (m3==0) {
		     nbm2 = l2 + 1;
		  }   
		  else {
		     nbm2 = l2+imin(l2,l1-m3)+1;         
		  }      
	          ga->w3j[n] = (double *)calloc(nbm2 ,sizeof(double));      
	          for (im2=0;im2<nbm2;im2++) {
	            fscanf(fin,"%lf",&x);
		    ga->w3j[n][im2] = x; 
	          }	  
	       }  
	    }
	  }
       }
     }
   }
  fclose(fin);
  
  return ga;
}

double gaunt3(int l1, int l2, int l3, int m1, int m2, int m3, Gaunt_Coeffs_3 *GAUNT)
{
   int il1,il2,il3,im1,im2,im3,ex,ex2;
   unsigned long ind0,ind3;
   double x,y;
   /* nullity cases */
   if (l1 >l2+l3 || l2>l1+l3  || l3>l1+l2 || (l1+l2+l3)%2!=0 || m1+m2+m3!=0 || abs(m1)>l1 || abs(m2)>l2 || abs(m3)>l3 )
      return 0;
   /* reduction function */
   /* permutation of (l1,m1), (l2,m2), (l3,m3) to get il1<=il2<=il3 */
   if (l1<l2) {
      if (l2<l3) { /* l1 < l2 < l3 */
        il1 = l3; im1 = m3; il2 = l2; im2 = m2; il3 = l1; im3 = m1;
      }
      else {
         if (l1<l3) { /* l1 < l3 <= l2 */
	    il1 = l2; im1 = m2; il2 = l3; im2 = m3; il3 = l1; im3 = m1;
	 }
	 else { /* l3 <= l1 < l2 */
            il1 = l2; im1 = m2; il2 = l1; im2 = m1; il3 = l3; im3 = m3;
	 }
      }
   }
   else {
      if (l2<l3) {
          if (l1<l3) { /* l2 <= l1 < l3 */
	     il1 = l3; im1 = m3; il2 = l1; im2 = m1; il3 = l2; im3 = m2;
	  }
	  else {  /* l2 < l3 <= l1 */
	     il1 = l1; im1 = m1; il2 = l3; im2 = m3; il3 = l2; im3 = m2;
	  }
      }
      else { /* l3 <= l2 <= l1 */
         il1 = l1; im1 = m1; il2 = l2; im2 = m2; il3 = l3; im3 = m3;
     }
   }
   
   /* first factor */
   if (il1%2==0)
      ind0 = il1*(il1-1)*(il1+10) - 12*il1*il2 + 12*il2*(il2+2) + 12*il3 + 24;
   else
      ind0 = il1*((il1+9)*il1-1) - 12*il1*il2 + 12*il2*(il2+1) + 12*il3 + 27;
   ind0 = ind0/24; 
   x = GAUNT->coef0[ind0-1];      
   /* symmetry on (im1,im2,im3) to get im3 >=0 */
   if (im3<0) {
     im1 = -im1;
     im2 = -im2;
     im3 = -im3;
   }  
   /* Regge symmetry */
   if ( (im3<il1-il2) && ((il1+il2+im3)%2==0)) {
     ex  = (il2-il1-im2+im1)/2;
     im2 = (il2-il1+im2-im1)/2;
     im1 = ex;
     ex2 = il1-il2;
     ex  = (il1+il2+im3)/2;
     il2 = (il1+il2-im3)/2;
     il1 = ex;
     im3 = ex2;
   }
   /* other symmetries */
   if (il1==il2) {
       if (im2>-iceil(im3,2)) {
          ex = im2; im2 = im1; im1 = ex;
       }	  
   }    
   else if (im3==0) {
       if (im2>0) {
          im1 = -im1; im2 = -im2;
       }
   }    
   /* End of reduction function */
   /* Index function using Regge symmetry f3(il1,il2,il3,im3) */   
   if (il1%2==0) {
     ind3 = il1*( ((5*il1+44)*il1 + 124)*il1 - 80) + 16*il2*(il2+2)*(4*il2+7)- 48*il1*il2*(il2+4) 
             - 96*il3*(il1-il2-il3) + 192*(im3+1) ;
     ind3 = ind3/48;
     if (il2%2!=0) ind3 = ind3 - (il1 - 2*il3 -3);
   }
   else {
     ind3 = ( ((5*il1+44)*il1 + 142)*il1 +76)*il1 + 405  + 32*il2*(il2+1)*(2*il2+1) - 48*il1*il2*(il2+2) 
             - 96*il3*(il1-il2-il3-1) + 192*im3;
     ind3 = ind3/48;
     if (il2%2!=0) ind3 = ind3 + (il1 - 2*il3 - 4);
}
   if (im3>=il1-il2) ind3 = ind3 - 4*(il1 - il2 - im3 - 1);
   if (il1%4==3) ind3 += 1;
   ind3 = ind3/8;

   y = GAUNT->w3j[ind3-1][il2+im2];
   
   return(x*y);  
}
