#include "rotation.h"

MatL *matl_calloc(int lmax) 
{
   int l;
   MatL *matl;
   matl = (MatL *)calloc(1,sizeof(MatL));
   matl->lmax = lmax;
   matl->lmat = (gsl_matrix **)calloc(lmax+1,sizeof(gsl_matrix *));
   for (l=0;l<=lmax;l++) {
      matl->lmat[l] = gsl_matrix_calloc(2*l+1,2*l+1); 
  }
   return matl;   
}

void  matl_free(MatL *matl) 
{
   int l;
   for (l=0;l<=matl->lmax;l++)
     gsl_matrix_free(matl->lmat[l]);
   free(matl->lmat);
   free(matl); 
}

int   matl_product(const MatL *matl1, const MatL *matl2, MatL *matres)
{
   int l;
   /* test sur les lmax !!! */
   for (l=0;l<=matl1->lmax;l++)
     gsl_blas_dgemm(CblasNoTrans,CblasNoTrans, 1.0, matl1->lmat[l], matl2->lmat[l], 0.0, matres->lmat[l]);
   return 0;
}

/* matl1 symmetric */
int   
matl_product_symm( CBLAS_SIDE_t Side, const MatL *matl1, 
                   const MatL *matl2, MatL *matres)
{
   int l;
/*printf("In matl_product_symm\n");   */
   /* test sur les lmax !!! */
   for (l=0;l<=matl2->lmax;l++) {
     gsl_blas_dsymm( Side, CblasLower, 1.0, matl1->lmat[l], 
                     matl2->lmat[l], 0.0, matres->lmat[l]);
   }

/*printf("Out matl_product_symm\n"); */
   return 0;
}

/* matl1 symmetric */
int   
matl_product_symm_n( CBLAS_SIDE_t Side, const MatL *matl1, 
                     const MatL *matl2, MatL *matres,
                     const unsigned int max_band)
{
   int l;
/*printf("In matl_product_symm\n");   */
   /* test sur les lmax !!! */
   for (l=0;l<=max_band;l++) {
     gsl_blas_dsymm( Side, CblasLower, 1.0, matl1->lmat[l], 
                     matl2->lmat[l], 0.0, matres->lmat[l]);
   }

/*printf("Out matl_product_symm\n"); */
   return 0;
}
