/*************************************/
/* Fichier: rotation.c               */
/* Version: 1.0                      */
/* Date:   January 2010              */
/* Auteur: Didier Pinchon            */
/*************************************/
#include "rotation.h"

MatL *read_lmatJ(int lmax)
{
   int l,k,k1,i,j,lmaxmax;
   double x;
   char com[50];
   MatL *matj;
   FILE *fin;
   
   if ((fin = fopen("data/numMatJ.dat","r")) == NULL) {
     fprintf(stderr,"Cannot open input file numMatJ.dat.\n");
   }
   
   matj = matl_calloc(lmax);
   
   fscanf(fin,"%s %d",com,&lmaxmax);
   if (lmax>lmaxmax) {
      fprintf(stderr,"lmax is too small in numMatJ.dat.\n");
   }
   
   // set matrix for band 0
   gsl_matrix_set(matj->lmat[0],0,0, 1.0);
   // set matrix for band 1
   gsl_matrix_set(matj->lmat[1],0,1, -1.0);
   gsl_matrix_set(matj->lmat[1],1,0, -1.0);
   gsl_matrix_set(matj->lmat[1],2,2, 1.0);

  for (l=2;l<=lmax;l++) {
    fscanf(fin,"%s %d",com,&lmaxmax);
    k = l/2;
    k1 = k;
    
    for (i=0;i<k;i++) {
      for (j=0;j<i;j++) {
        fscanf(fin,"%lf",&x);
        gsl_matrix_set(matj->lmat[l],2*i+1,2*j+1,x);
        gsl_matrix_set(matj->lmat[l],2*j+1,2*i+1,x);
      }
      fscanf(fin,"%lf",&x);
      gsl_matrix_set(matj->lmat[l],2*i+1,2*i+1,x);
    }

    if (l!=2*k1) k++;
    for (i=0;i<k;i++) {
      for (j=0;j<k;j++) {
        fscanf(fin,"%lf",&x);
        gsl_matrix_set(matj->lmat[l],2*k1+2*j+1,2*i,x);
        gsl_matrix_set(matj->lmat[l],2*i,2*k1+2*j+1,x);
      }
    }
    
    if (l==2*k1) 
      {k++;} 
    else 
      {k1++;}
    
    for (i=0;i<k;i++) {
      for (j=0;j<i;j++) {
        fscanf(fin,"%lf",&x);
        gsl_matrix_set(matj->lmat[l],2*k1+2*i,2*k1+2*j,x);
        gsl_matrix_set(matj->lmat[l],2*k1+2*j,2*k1+2*i,x);
      }
      fscanf(fin,"%lf",&x);
      gsl_matrix_set(matj->lmat[l],2*k1+2*i,2*k1+2*i,x);
    } 
  }

  fclose(fin);
  return matj;
}

int   matl_product_Zalpha_left(const MatL *matl, const double alpha,MatL *res) 
{
   int l,i,j;
   double co,si,x,y;
   extern double cos(double);
   extern double sin(double);

   for (l=1;l<=matl->lmax;l++)
      for (j=0;j<=2*l;j++) 
          gsl_matrix_set(res->lmat[l],l,j,gsl_matrix_get(matl->lmat[l],l,j));
        
   for (i=1;i<=matl->lmax;i++) {
       co = cos(i*alpha);
       si = sin(i*alpha);
       for (l=i;l<=matl->lmax;l++) {
    for (j=0;j<=2*l;j++) {
      x = gsl_matrix_get(matl->lmat[l],l-i,j);
      y = gsl_matrix_get(matl->lmat[l],l+i,j);
      gsl_matrix_set(res->lmat[l],l-i,j, co*x+si*y);
      gsl_matrix_set(res->lmat[l],l+i,j, -si*x+co*y);
    }     
       }
   }
   return EXIT_SUCCESS;
}

int   matl_product_Zalpha_left_lmax( const int lmax, const MatL *matl, const double alpha,MatL *res) 
{
   int l,i,j;
   double co,si,x,y;
   extern double cos(double);
   extern double sin(double);

   for (l=1;l<=lmax;l++)
      for (j=0;j<=2*l;j++) 
          gsl_matrix_set(res->lmat[l],l,j,gsl_matrix_get(matl->lmat[l],l,j));
        
   for (i=1;i<=lmax;i++) {
       co = cos(i*alpha);
       si = sin(i*alpha);
       for (l=i;l<=lmax;l++) {
    for (j=0;j<=2*l;j++) {
      x = gsl_matrix_get(matl->lmat[l],l-i,j);
      y = gsl_matrix_get(matl->lmat[l],l+i,j);
      gsl_matrix_set(res->lmat[l],l-i,j, co*x+si*y);
      gsl_matrix_set(res->lmat[l],l+i,j, -si*x+co*y);
    }     
       }
   }
   return EXIT_SUCCESS;
}

int   matl_product_Zalpha_right(const MatL *matl, const double alpha,MatL *res) 
{
   int l,i,j;
   double co,si,x,y;
   extern double cos(double);
   extern double sin(double);
   
   for (l=1;l<=matl->lmax;l++)
      for (i=0;i<=2*l;i++) 
          gsl_matrix_set(res->lmat[l],i,l,gsl_matrix_get(matl->lmat[l],i,l));
    
   for (j=1;j<=matl->lmax;j++) {
       co = cos(j*alpha);
       si = sin(j*alpha);
       for (l=j;l<=matl->lmax;l++) {
    for (i=0;i<=2*l;i++) {
      x = gsl_matrix_get(matl->lmat[l],i,l-j);
      y = gsl_matrix_get(matl->lmat[l],i,l+j);
      gsl_matrix_set(res->lmat[l],i,l-j, co*x-si*y);
      gsl_matrix_set(res->lmat[l],i,l+j, si*x+co*y);
    }     
       }
   }
   return EXIT_SUCCESS;
}

int   matl_product_Zalpha_right_lmax( const int lmax, const MatL *matl, const double alpha,MatL *res) 
{
   int l,i,j;
   double co,si,x,y;
   extern double cos(double);
   extern double sin(double);
   
   for (l=1;l<=lmax;l++)
      for (i=0;i<=2*l;i++) 
          gsl_matrix_set(res->lmat[l],i,l,gsl_matrix_get(matl->lmat[l],i,l));
    
   for (j=1;j<=lmax;j++) {
       co = cos(j*alpha);
       si = sin(j*alpha);
       for (l=j;l<=lmax;l++) {
    for (i=0;i<=2*l;i++) {
      x = gsl_matrix_get(matl->lmat[l],i,l-j);
      y = gsl_matrix_get(matl->lmat[l],i,l+j);
      gsl_matrix_set(res->lmat[l],i,l-j, co*x-si*y);
      gsl_matrix_set(res->lmat[l],i,l+j, si*x+co*y);
    }     
       }
   }
   return EXIT_SUCCESS;
}

int   matl_product_symm_lmax( const int lmax, CBLAS_SIDE_t Side,const MatL *matl1, const MatL *matl2, MatL *matres)
{
   int l;
/*printf("In matl_product_symm\n");   */
   /* test sur les lmax !!! */
   for (l=0;l<=lmax;l++)
     gsl_blas_dsymm(Side, CblasLower, 1.0, matl1->lmat[l], matl2->lmat[l], 0.0, matres->lmat[l]);
/*printf("Out matl_product_symm\n"); */
   return 0;
}

int matl_rotation(int lmax,double alpha, double beta, double gamma,MatL *res,const MatL *matj)
{
  MatL *tmp;
  
  tmp = matl_calloc(lmax);
  
  matl_product_Zalpha_left_lmax(lmax,matj,beta,tmp);
  matl_product_symm_lmax(lmax,CblasLeft ,matj, tmp, res);
  matl_product_Zalpha_left_lmax(lmax,res,alpha,tmp);
  matl_product_Zalpha_right_lmax(lmax,tmp,gamma,res);
  
  matl_free(tmp);
  return EXIT_SUCCESS;
}

gsl_matrix *read_matJ(int l)
{
   int k,k1,i,j, lval;
   double x;
   char com[50] ,fname[80];
   gsl_matrix *matj;
   FILE *fin;

   sprintf(fname,"data/pinchon/numMatJ-%d.dat",l);
 
   if ((fin = fopen(fname,"r")) == NULL) {
     fprintf(stderr,"Cannot open input file %s.\n",fname);
   }
   
   matj = gsl_matrix_calloc(2*l+1,2*l+1); 

   fscanf(fin,"%s %d",com,&lval);
   if (lval!=l) {
      fprintf(stderr,"bad value of l in %.\n",fname);
   }

   if (l==0) {
     gsl_matrix_set(matj,0,0, 1.0);
     return matj;
   }
   if (l==1) {
     gsl_matrix_set(matj,0,1, -1.0);
     gsl_matrix_set(matj,1,0, -1.0);
     gsl_matrix_set(matj,2,2, 1.0);
     return matj;
   }
   
   k = l/2;
   k1 = k;
   for (i=0;i<k;i++) {
         for (j=0;j<i;j++) {
      fscanf(fin,"%lf",&x);
            gsl_matrix_set(matj,2*i+1,2*j+1,x);
      gsl_matrix_set(matj,2*j+1,2*i+1,x);
   }
   fscanf(fin,"%lf",&x);
   gsl_matrix_set(matj,2*i+1,2*i+1,x);
   }
   if (l!=2*k1) k++;
   for (i=0;i<k;i++) {
      for (j=0;j<k;j++) {
      fscanf(fin,"%lf",&x);
            gsl_matrix_set(matj,2*k1+2*j+1,2*i,x);
      gsl_matrix_set(matj,2*i,2*k1+2*j+1,x);
         }
    }
    if (l==2*k1) {k++;} else {k1++;}
    for (i=0;i<k;i++) {
         for (j=0;j<i;j++) {
      fscanf(fin,"%lf",&x);
            gsl_matrix_set(matj,2*k1+2*i,2*k1+2*j,x);
      gsl_matrix_set(matj,2*k1+2*j,2*k1+2*i,x);
   }
   fscanf(fin,"%lf",&x);
   gsl_matrix_set(matj,2*k1+2*i,2*k1+2*i,x);
   } 
   fclose(fin);
   return matj;
}

int   mat_product_Zalpha_left(int l,const gsl_matrix *mat, const double alpha,gsl_matrix *res) 
{
   int i,j;
   double co,si,x,y;
   extern double cos(double);
   extern double sin(double);

   for (j=0;j<=2*l;j++) 
       gsl_matrix_set(res,l,j,gsl_matrix_get(mat,l,j));
        
   for (i=1;i<=l;i++) {
       co = cos(i*alpha);
       si = sin(i*alpha);
    for (j=0;j<=2*l;j++) {
      x = gsl_matrix_get(mat,l-i,j);
      y = gsl_matrix_get(mat,l+i,j);
      gsl_matrix_set(res,l-i,j, co*x+si*y);
      gsl_matrix_set(res,l+i,j, -si*x+co*y);
    }     
   }
   return EXIT_SUCCESS;
}

int mat_product_Zalpha_right(int l,const gsl_matrix  *mat, const double alpha,gsl_matrix  *res) 
{
   int i,j;
   double co,si,x,y;
   extern double cos(double);
   extern double sin(double);
   

   for (i=0;i<=2*l;i++) 
       gsl_matrix_set(res,i,l,gsl_matrix_get(mat,i,l));
    
   for (j=1;j<=l;j++) {
       co = cos(j*alpha);
       si = sin(j*alpha);
    for (i=0;i<=2*l;i++) {
      x = gsl_matrix_get(mat,i,l-j);
      y = gsl_matrix_get(mat,i,l+j);
      gsl_matrix_set(res,i,l-j, co*x-si*y);
      gsl_matrix_set(res,i,l+j, si*x+co*y);
    }     
   }
   return EXIT_SUCCESS;
}

int mat_rotation(int l,double alpha, double beta, double gamma,gsl_matrix *res,const gsl_matrix *matj)
{
  gsl_matrix *tmp;
  tmp = gsl_matrix_calloc(2*l+1,2*l+1); 
  
  mat_product_Zalpha_left(l,matj,beta,tmp);
  /* gsl_blas_dsymm(CblasLeft, CblasLower, 1.0, matj, tmp, 0.0, res); */
  gsl_blas_dgemm(CblasNoTrans,CblasNoTrans,1.0,matj,tmp,0.0,res);
  mat_product_Zalpha_left(l,res,alpha,tmp);
  mat_product_Zalpha_right(l,tmp,gamma,res);
  
  gsl_matrix_free(tmp);
  return EXIT_SUCCESS;
}

/*
int mat_rotation(int l,double alpha, double beta, double gamma,gsl_matrix *res, gsl_matrix *matj,
   gsl_matrix *tmp)
{ 
  mat_product_Zalpha_left(l,matj,beta,tmp);
  gsl_blas_dsymm(CblasLeft, CblasLower, 1.0, matj, tmp, 0.0, res);
  mat_product_Zalpha_left(l,res,alpha,tmp);
  mat_product_Zalpha_right(l,tmp,gamma,res);

  return EXIT_SUCCESS;
}
*/
