% load file has to be performed manually since the results file do have
% a date stamp in the name. Performing a 'clear' before calling the 
% function is advised

% initialize
algos = { {'ShrK, osr = 1.0, nested', 1}, ...
          {'ShrK, osr = 1.0, zonal', 2}, ...
          {'ShrK, osr = 1.0, opt.', 3}, ...
          {'ShrK, osr = 1.1, opt.', 4}, ...
          {'ShrK, osr = 2.0, opt.', 5}, ...
          {'Equi-latitude', 6}, ...
          {'Pinchon (Matrix)', 7}, ...
          {'Pinchon (Vector)', 8}, ...
          {'Ivanic', 9}, ...
          {'Blanco', 10}, ...
          {'Gimbutas', 11} };
leg_names = {};
for( i = 1 : numel( algos))
  leg_names{i} = algos{i}{1};
end

num_algos = numel( algos);
num_bands = size( exec_times, 2);

% plotting parameters
colors = {[1 0 0], [0 1 0], [0 0 1], [1 0.5 0], [0 1 1], [0 0 0],...
          [1 0 1], [1 0.5 0], [0 1 0], [0 0 1], [0 0 0.5], };          
line_style = {'-', '-', '-', '-', '-', '-',...
              '--', '--', '--', '--', '-.'};

%% Plot execution time
figure;
hold on;
for( i = 1 : num_algos)
  plot( 1:num_bands, exec_times(algos{i}{2},:), ...
        'Color', colors{i}, 'LineStyle', line_style{i}, 'MarkerSize', 10);
end
title( 'Execution Time');
legend( leg_names, 'Location', 'SouthEast', 'FontSize', 8.0);
xlabel('Band')
ylabel('Time');
xlim( [1,num_bands]);
set( gca, 'YScale', 'log');
set( gca, 'YAxisLocation', 'right');

%% Plot execution time (detail)
if( num_bands > 10)

x_range = [1:10];

figure;
hold on;
for( i = 1 : num_algos)
  plot( x_range, exec_times(algos{i}{2},x_range), ...
        'Color', colors{i}, 'LineStyle', line_style{i}, 'MarkerSize', 10);
end
title( 'Execution Time (detail)');
legend( leg_names, 'Location', 'SouthEast', 'FontSize', 8.0);
xlabel('Band')
ylabel('Time');
set( gca, 'YScale', 'log');
axis( 'tight');
set( gca, 'YAxisLocation', 'right');

end

%% Plot average L1 error
figure;
hold on;
for( i = 1 : num_algos)
  plot( 1:num_bands, err_avg_l1(algos{i}{2},:), ...
        'Color', colors{i}, 'LineStyle', line_style{i}, 'MarkerSize', 10);
end
title( 'Average L_1 Error');
legend( leg_names, 'Location', 'NorthWest', 'FontSize', 8.0);
xlabel('Band')
ylabel('Error');
set( gca, 'YScale', 'log');
xlim( [1,num_bands]);
axis( 'tight');
set( gca, 'YAxisLocation', 'right');

%% Plot average L2 error
figure;
hold on;
for( i = 1 : num_algos)
  plot( 1:num_bands, err_avg_l2(algos{i}{2},:), ...
        'Color', colors{i}, 'LineStyle', line_style{i}, 'MarkerSize', 10);
end
title( 'Average L_2 Error');
legend( leg_names, 'Location', 'NorthWest', 'FontSize', 8.0);
xlabel('Band')
ylabel('Error');
set( gca, 'YScale', 'log');
xlim( [1,num_bands]);
axis( 'tight');
set( gca, 'YAxisLocation', 'right');

%% Plot average L_infty error
figure;
hold on;
for( i = 1 : num_algos)
  plot( 1:num_bands, err_avg_linfty(algos{i}{2},:), ...
        'Color', colors{i}, 'LineStyle', line_style{i}, 'MarkerSize', 10);
end
title( 'Average L_{\infty} Error');
legend( leg_names, 'Location', 'NorthWest', 'FontSize', 8.0);
xlabel('Band')
ylabel('Error');
set( gca, 'YScale', 'log');
xlim( [1,num_bands]);
axis( 'tight');
set( gca, 'YAxisLocation', 'right');

% %% Plot maximum L1 error
% figure;
% hold on;
% for( i = 1 : num_algos)
%   plot( 1:num_bands, err_max_l1(algos{i}{2},:), ...
%         'Color', colors{i}, 'LineStyle', line_style{i}, 'MarkerSize', 10);
% end
% title( 'Maximum L_1 Error');
% legend( leg_names, 'Location', 'NorthWest', 'FontSize', 8.0);
% xlabel('Band')
% ylabel('Error');
% set( gca, 'YScale', 'log');
% xlim( [1,num_bands]);
% axis( 'tight');
% set( gca, 'YAxisLocation', 'right');
% 
% %% Plot maximum L2 error
% figure;
% hold on;
% for( i = 1 : num_algos)
%   plot( 1:num_bands, err_max_l2(algos{i}{2},:), ...
%         'Color', colors{i}, 'LineStyle', line_style{i}, 'MarkerSize', 10);
% end
% title( 'Maximum L_2 Error');
% legend( leg_names, 'Location', 'NorthWest', 'FontSize', 8.0);
% xlabel('Band')
% ylabel('Error');
% set( gca, 'YScale', 'log');
% xlim( [1,num_bands]);
% axis( 'tight');
% set( gca, 'YAxisLocation', 'right');
% 
% %% Plot maximum L_infty error
% figure;
% hold on;
% for( i = 1 : num_algos)
%   plot( 1:num_bands, err_max_linfty(algos{i}{2},:), ...
%         'Color', colors{i}, 'LineStyle', line_style{i}, 'MarkerSize', 10);
% end
% title( 'Maximum L_{\infty} Error');
% legend( leg_names, 'Location', 'NorthWest', 'FontSize', 8.0);
% xlabel('Band')
% ylabel('Error');
% set( gca, 'YScale', 'log');
% xlim( [1,num_bands]);
% axis( 'tight');
% set( gca, 'YAxisLocation', 'right');