% load file has to be performed manually since the results file do have
% a date stamp in the name. Performing a 'clear' before calling the 
% function is advised

% initialize
algos = { {'ShrK, osr = 1.0', 1}, ...
          {'ShrK, osr = 1.0, zonal', 2}, ...
          {'ShrK, osr = 1.0, opt.', 3}, ...
          {'ShrK, osr = 1.1, opt.', 4}, ...
          {'ShrK, osr = 2.0, opt.', 5}, ...
          {'Equi-latitude', 6}, ...
          {'Pinchon', 7}, ...
          {'Ivanic', 8}, ...
          {'Blanco', 9}, ...
          {'Gimbutas', 10} };
leg_names = {};
for( i = 1 : numel( algos))
  leg_names{i} = algos{i}{1};
end

% basic parameters
num_algos = numel( algos);
num_bands = size( errs{1}{1}, 1);

% plotting parameters
colors = {[1 0 0], [0 1 0], [0 0 1], [1 0.5 0], [0 1 1], [0 0 0],...
          [1 0 1], [1 0.5 0], [0 1 0], [0 0 1] };          
line_style = {'-', '-', '-', '-', '-', '-',...
              '--', '--', '--', '--'};
 
% % excluse rotations where the Blanco algorithm generated NaNs
% [row,col] = find( isnan(errs{8}{2}));
idx = [1:size( rotations, 1)];
% idx = setdiff( idx, unique(col));
%             
            
% use at least machine epsilon for all values to avoid problems with log
% plots
% do for all methods
for algo = 1:numel(errs)
  % do for all bands
  for band = 1:num_bands

    % max l1 norm
    err_max_l1(algo, band) =  max( eps, max( errs{algo}{1}(band,idx)));          
    % avg norm of the l2 norm           
    err_avg_l1(algo, band) = max( eps, mean( errs{algo}{1}(band,idx))); 
 
    % max l2 norm
    err_max_l2(algo, band) = max( eps, max( errs{algo}{2}(band,idx)));          
    % avg norm of the l2 norm           
    err_avg_l2(algo, band) = max( eps, mean( errs{algo}{2}(band,idx))); 
   
    % max l_infty norm
    err_max_linfty(algo, band) = max( eps, max( errs{algo}{3}(band,idx)));          
    % avg norm of the l2 norm           
    err_avg_linfty(algo, band) = max( eps, mean( errs{algo}{3}(band,idx))); 
  
  end
end

%% Plot average L1 error
figure;
hold on;
for( i = 1 : num_algos)
  plot( 0:num_bands-1, err_avg_l1(algos{i}{2},:), ...
        'Color', colors{i}, 'LineStyle', line_style{i}, 'MarkerSize', 10);
end
title( 'Average L_1 Error');
legend( leg_names, 'Location', 'NorthWest');
xlabel('Band')
ylabel('Error');
set( gca, 'YScale', 'log');
xlim( [0,num_bands]);
axis( 'tight');
set( gca, 'YAxisLocation', 'right');

%% Plot average L2 error
figure;
hold on;
for( i = 1 : num_algos)
  plot( 0:num_bands-1, err_avg_l2(algos{i}{2},:), ...
        'Color', colors{i}, 'LineStyle', line_style{i}, 'MarkerSize', 10);
end
title( 'Average L_2 Error');
legend( leg_names, 'Location', 'NorthWest');
xlabel('Band')
ylabel('Error');
set( gca, 'YScale', 'log');
xlim( [0,num_bands]);
axis( 'tight');
set( gca, 'YAxisLocation', 'right');

%% Plot average L_infty error
figure;
hold on;
for( i = 1 : num_algos)
  plot( 0:num_bands-1, err_avg_linfty(algos{i}{2},:), ...
        'Color', colors{i}, 'LineStyle', line_style{i}, 'MarkerSize', 10);
end
title( 'Average L_{\infty} Error');
legend( leg_names, 'Location', 'NorthWest');
xlabel('Band')
ylabel('Error');
set( gca, 'YScale', 'log');
xlim( [0,num_bands]);
axis( 'tight');
set( gca, 'YAxisLocation', 'right');

%% Plot maximum L1 error
figure;
hold on;
for( i = 1 : num_algos)
  plot( 0:num_bands-1, err_max_l1(algos{i}{2},:), ...
        'Color', colors{i}, 'LineStyle', line_style{i}, 'MarkerSize', 10);
end
title( 'Maximum L_1 Error');
legend( leg_names, 'Location', 'NorthWest');
xlabel('Band')
ylabel('Error');
set( gca, 'YScale', 'log');
xlim( [0,num_bands]);
axis( 'tight');
set( gca, 'YAxisLocation', 'right');

%% Plot maximum L2 error
figure;
hold on;
for( i = 1 : num_algos)
  plot( 0:num_bands-1, err_max_l2(algos{i}{2},:), ...
        'Color', colors{i}, 'LineStyle', line_style{i}, 'MarkerSize', 10);
end
title( 'Maximum L_2 Error');
legend( leg_names, 'Location', 'NorthWest');
xlabel('Band')
ylabel('Error');
set( gca, 'YScale', 'log');
xlim( [0,num_bands]);
axis( 'tight');
set( gca, 'YAxisLocation', 'right');

%% Plot maximum L_infty error
figure;
hold on;
for( i = 1 : num_algos)
  plot( 0:num_bands-1, err_max_linfty(algos{i}{2},:), ...
        'Color', colors{i}, 'LineStyle', line_style{i}, 'MarkerSize', 10);
end
title( 'Maximum L_{\infty} Error');
legend( leg_names, 'Location', 'NorthWest');
xlabel('Band')
ylabel('Error');
set( gca, 'YScale', 'log');
xlim( [0,num_bands]);
axis( 'tight');
set( gca, 'YAxisLocation', 'right');
