/// @file
////////////////////////////////////////////////////////////////////////////////
///
/// Copyright (C) 2009      University of Toronto
///
////////////////////////////////////////////////////////////////////////////////
///
///  module     : main
///
///  author     : {lessig|tyler}@dgp.toronto.edu
///
///  project    : Spherical Harmonics rotation
///
///  description: Project main file
///
///  license    : See README for details.
///
////////////////////////////////////////////////////////////////////////////////

// includes, system
#include <iostream>
#include <cstdlib>
#include <vector>
#include <fstream>
#include <iomanip>

// global typedefs
typedef  double FType;

// includes, project
#include <config.h>
#include <util.h>
#include <cutil.h>
#include <sh_rotation_benchmark.h>
#include <cmd_line_args.h>

////////////////////////////////////////////////////////////////////////////////
//! Program main
////////////////////////////////////////////////////////////////////////////////
int
main( int argc, char** argv) {

  // Typedefs
  typedef  ShCoeffs<FType>  ShCoeffsF;

  int is_accuracy_test = 0;
  unsigned int num_rand_rotations = 75;
  unsigned int num_iterations_timing = 10;
  unsigned int max_band = 1;

  // check for command line arguments
  getCmdLineArgMaxBand( argc, argv, max_band);
  getCmdLineArgNumRotations( argc, argv, num_rand_rotations);
  getCmdLineArgNumItersTiming( argc, argv, num_iterations_timing);
  getCmdLineArgAccuracy( argc, argv, is_accuracy_test);

  ShEvaluator<FType>* sh_evaluator = ShEvaluator<FType>::instance();
  sh_evaluator->init( max_band);

  ShRotationBenchmark<FType> rot_bench;

  if (is_accuracy_test) {
    // When '--accuracy=1' is passed on command line, the following 
    // performs an accuracy test.  Does not preload matrices,
    // does not measure performance.
    printf( "Accuracy Test\n");

    // run test
    rot_bench.runExperimentAccuracy( max_band, num_rand_rotations, 0);
    
    // write result to disk
    rot_bench.writeResultsMatlabAccuracy();
  } 
  else {
    // By default, perform a a performance test.  Matrices are preloaded
    printf( "Performance Test\n");

    // run test
    rot_bench.runExperiment( max_band, num_rand_rotations,
                             num_iterations_timing);

    // write result to disk
    rot_bench.writeResultsMatlab();
  }

  return EXIT_SUCCESS;
}

